@@ -1,5 +1,5 @@
 """Package to detect spindles, ripples, slow waves.
 """
-from .spindle import DetectSpindle, merge_close
+from .spindle import DetectSpindle, merge_close, transform_signal
 from .ripple import DetectRipple
 from .slowwave import DetectSlowWave
