@@ -5,6 +5,7 @@ from logging import getLogger
 
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import (QComboBox,
+                             QErrorMessage,
                              QFormLayout,
                              QGroupBox,
                              QHBoxLayout,
@@ -12,6 +13,7 @@ from PyQt5.QtWidgets import (QComboBox,
                              )
 
 from ..detect import DetectSpindle, DetectSlowWave
+from ..trans import fetch_segments
 from .modal_widgets import ChannelDialog
 from .notes import SPINDLE_METHODS, SLOW_WAVE_METHODS
 from .utils import FormStr, FormFloat, FormBool, FormMenu
@@ -51,22 +53,23 @@ class SpindleDialog(ChannelDialog):
         self.idx_group.activated.connect(self.update_channels)
         self.idx_chan.itemSelectionChanged.connect(self.count_channels)
 
-        flayout = QFormLayout(box0)
-        flayout.addRow('Label',
+        form = QFormLayout(box0)
+        form.addRow('Label',
                        self.label)
-        flayout.addRow('Channel group',
+        form.addRow('Channel group',
                        self.idx_group)
-        flayout.addRow('Channel(s)',
+        form.addRow('Channel(s)',
                        self.idx_chan)
-        flayout.addRow('Cycle(s)',
+        form.addRow('Cycle(s)',
                        self.idx_cycle)
-        flayout.addRow('Stage(s)',
+        form.addRow('Stage(s)',
                        self.idx_stage)
 
         box1 = QGroupBox('Parameters')
 
         self.index['f1'] = FormFloat()
         self.index['f2'] = FormFloat()
+        self.index['rolloff'] = FormFloat()
         self.index['min_dur'] = FormFloat()
         self.index['max_dur'] = FormFloat()
         self.index['sigma'] = FormFloat()
@@ -81,44 +84,52 @@ class SpindleDialog(ChannelDialog):
         self.method = self.idx_method.currentText()
         self.idx_method.currentIndexChanged.connect(self.update_values)
 
-        flayout = QFormLayout(box1)
-        flayout.addRow('Method',
+        form = QFormLayout(box1)
+        form.addRow('Method',
                        self.idx_method)
-        flayout.addRow('Lowcut (Hz)',
+        form.addRow('Lowcut (Hz)',
                        self.index['f1'])
-        flayout.addRow('Highcut (Hz)',
+        form.addRow('Highcut (Hz)',
                        self.index['f2'])
-        flayout.addRow('Min. duration (sec)',
+        form.addRow('Roll-off (Hz)',
+                       self.index['rolloff'])
+        form.addRow('Min. duration (sec)',
                        self.index['min_dur'])
-        flayout.addRow(' Max. duration (sec)',
+        form.addRow(' Max. duration (sec)',
                        self.index['max_dur'])
-        flayout.addRow('Wavelet sigma',
+        form.addRow('Wavelet sigma',
                        self.index['sigma'])
-        flayout.addRow('Detection window',
+        form.addRow('Detection window',
                        self.index['win_sz'])
-        flayout.addRow('Smoothing',
+        form.addRow('Smoothing',
                        self.index['smooth'])
-        flayout.addRow('Detection threshold, low',
+        form.addRow('Detection threshold, low',
                        self.index['det_thresh_lo'])
-        flayout.addRow('Detection threshold, high',
+        form.addRow('Detection threshold, high',
                        self.index['det_thresh_hi'])
-        flayout.addRow('Selection threshold',
+        form.addRow('Selection threshold',
                        self.index['sel_thresh'])
-        flayout.addRow('Min. interval',
+        form.addRow('Min. interval',
                        self.index['interval'])
 
         box3 = QGroupBox('Options')
 
         self.index['merge'] = FormBool('Merge events across channels')
-        self.index['exclude'] = FormBool('Exclude Artefact events')
+        self.index['excl_epoch'] = FormBool('Exclude Poor signal epochs')
+        self.index['excl_event'] = FormBool('Exclude Artefact events')
+        self.index['min_seg_dur'] = FormFloat(5)
 
+        self.index['excl_epoch'].set_value(True)
+        self.index['excl_event'].set_value(True)
         self.index['merge'].setCheckState(Qt.Unchecked)
         self.index['merge'].setEnabled(False)
-        self.index['exclude'].set_value(True)
 
-        flayout = QFormLayout(box3)
-        flayout.addRow(self.index['exclude'])
-        flayout.addRow(self.index['merge'])
+        form = QFormLayout(box3)
+        form.addRow(self.index['excl_epoch'])
+        form.addRow(self.index['excl_event'])
+        form.addRow('Minimum subsegment duration',
+                       self.index['min_seg_dur'])
+        form.addRow(self.index['merge'])
 
         self.bbox.clicked.connect(self.button_clicked)
 
@@ -150,9 +161,11 @@ class SpindleDialog(ChannelDialog):
         if button is self.idx_ok:
 
             chans = self.get_channels()
+            group = self.one_grp
             cycle = self.get_cycles()
             stage = self.idx_stage.selectedItems()
             params = {k: v.get_value() for k, v in self.index.items()}
+            label = self.label.get_value()
 
             if None in [params['f1'], params['f2']]:
                 self.parent.statusBar().showMessage(
@@ -172,12 +185,29 @@ class SpindleDialog(ChannelDialog):
             else:
                 stage = [x.text() for x in self.idx_stage.selectedItems()]
 
-            self.parent.notes.read_data(chans, self.one_grp, period=cycle,
-                                        stage=stage, qual='Good',
-                                        exclude_artf=params['exclude'])
-            if self.parent.notes.data is not None:
-                self.parent.notes.detect_events(self.method, params,
-                                                label=self.label.get_value())
+            data = fetch_segments(self.parent.info.dataset, 
+                                  self.parent.notes.annot, cat=(1, 1, 1, 0),
+                                  stage=stage, cycle=cycle, 
+                                  min_dur=params['min_seg_dur'], 
+                                  reject_epoch=params['excl_epoch'], 
+                                  reject_artf=params['excl_event'])
+            
+            if not data.segments:
+                msg = 'No valid signal found.'
+                error_dialog = QErrorMessage(self)
+                error_dialog.setWindowTitle('Error fetching data')
+                error_dialog.showMessage(msg)
+                return
+            
+            ding = data.read_data(chans, group['ref_chan'], group['name'],
+                                  parent=self)
+            
+            if not ding:
+                self.parent.statusBar().showMessage('Process interrupted.')
+                return            
+            
+            self.parent.notes.detect_events(data[0]['data'], self.method, 
+                                            params, label=label)
 
             self.accept()
 
@@ -196,6 +226,7 @@ class SpindleDialog(ChannelDialog):
 
         self.index['f1'].set_value(spin_det.frequency[0])
         self.index['f2'].set_value(spin_det.frequency[1])
+        self.index['rolloff'].set_value(spin_det.rolloff)
         self.index['min_dur'].set_value(spin_det.duration[0])
         self.index['max_dur'].set_value(spin_det.duration[1])
         self.index['sigma'].set_value(spin_det.det_wavelet['sd'])
@@ -206,7 +237,7 @@ class SpindleDialog(ChannelDialog):
         self.index['interval'].set_value(spin_det.min_interval)
 
         for param in ['sigma', 'win_sz', 'det_thresh_lo', 'det_thresh_hi',
-                      'sel_thresh', 'smooth']:
+                      'sel_thresh', 'smooth', 'rolloff']:
             widg = self.index[param]
             if widg.get_value() == 0:
                 widg.set_value('N/A')
@@ -259,17 +290,16 @@ class SWDialog(ChannelDialog):
         self.label.setText('sw')
         self.idx_group.activated.connect(self.update_channels)
 
-        flayout = QFormLayout()
-        box0.setLayout(flayout)
-        flayout.addRow('Label',
+        form = QFormLayout(box0)
+        form.addRow('Label',
                            self.label)
-        flayout.addRow('Channel group',
+        form.addRow('Channel group',
                        self.idx_group)
-        flayout.addRow('Channel(s)',
+        form.addRow('Channel(s)',
                        self.idx_chan)
-        flayout.addRow('Cycle(s)',
+        form.addRow('Cycle(s)',
                        self.idx_cycle)
-        flayout.addRow('Stage(s)',
+        form.addRow('Stage(s)',
                        self.idx_stage)
 
         box1 = QGroupBox('Parameters')
@@ -291,38 +321,46 @@ class SWDialog(ChannelDialog):
         self.index['min_dur'] = FormFloat()
         self.index['max_dur'] = FormFloat()
 
-        flayout = QFormLayout(box1)
-        flayout.addRow('Method',
+        form = QFormLayout(box1)
+        form.addRow('Method',
                             mbox)
-        flayout.addRow('Lowcut (Hz)',
+        form.addRow('Lowcut (Hz)',
                            self.index['f1'])
-        flayout.addRow('Highcut (Hz)',
+        form.addRow('Highcut (Hz)',
                            self.index['f2'])
-        flayout.addRow('Min. trough duration (sec)',
+        form.addRow('Min. trough duration (sec)',
                            self.index['min_trough_dur'])
-        flayout.addRow(' Max. trough duration (sec)',
+        form.addRow(' Max. trough duration (sec)',
                            self.index['max_trough_dur'])
-        flayout.addRow(' Max. trough amplitude (uV)',
+        form.addRow(' Max. trough amplitude (uV)',
                            self.index['max_trough_amp'])
-        flayout.addRow('Min. peak-to-peak amplitude (uV)',
+        form.addRow('Min. peak-to-peak amplitude (uV)',
                            self.index['min_ptp'])
-        flayout.addRow('Min. duration (sec)',
+        form.addRow('Min. duration (sec)',
                            self.index['min_dur'])
-        flayout.addRow(' Max. duration (sec)',
+        form.addRow(' Max. duration (sec)',
                            self.index['max_dur'])
         box3 = QGroupBox('Options')
 
         self.index['demean'] = FormBool('De-mean (by channel mean)')
         self.index['exclude'] = FormBool('Exclude Artefact events')
         self.index['invert'] = FormBool('Invert detection')
+        self.index['excl_epoch'] = FormBool('Exclude Poor signal epochs')
+        self.index['excl_event'] = FormBool('Exclude Artefact events')
+        self.index['min_seg_dur'] = FormFloat(5)
 
+        self.index['excl_epoch'].set_value(True)
+        self.index['excl_event'].set_value(True)
         self.index['demean'].set_value(True)
         self.index['exclude'].set_value(True)
 
-        flayout = QFormLayout(box3)
-        flayout.addRow(self.index['demean'])
-        flayout.addRow(self.index['exclude'])
-        flayout.addRow(self.index['invert'])
+        form = QFormLayout(box3)
+        form.addRow(self.index['excl_epoch'])
+        form.addRow(self.index['excl_event'])
+        form.addRow('Minimum subsegment duration',
+                       self.index['min_seg_dur'])
+        form.addRow(self.index['demean'])
+        form.addRow(self.index['invert'])
 
         self.bbox.clicked.connect(self.button_clicked)
 
@@ -354,9 +392,11 @@ class SWDialog(ChannelDialog):
         if button is self.idx_ok:
 
             chans = self.get_channels()
+            group = self.one_grp
             cycle = self.get_cycles()
             stage = self.idx_stage.selectedItems()
             params = {k: v.get_value() for k, v in self.index.items()}
+            label = self.label.get_value()
 
             if None in [params['f1'], params['f2']]:
                 self.parent.statusBar().showMessage(
@@ -368,14 +408,29 @@ class SWDialog(ChannelDialog):
             else:
                 stage = [x.text() for x in self.idx_stage.selectedItems()]
 
-            self.parent.notes.read_data(chans, self.one_grp, period=cycle,
-                                        stage=stage, qual='Good',
-                                        demean=params['demean'],
-                                        exclude_artf=params['exclude'])
-
-            if self.parent.notes.data is not None:
-                self.parent.notes.detect_events(self.method, params,
-                                                label=self.label.get_value())
+            data = fetch_segments(self.parent.info.dataset, 
+                                  self.parent.notes.annot, cat=(1, 1, 1, 0),
+                                  stage=stage, cycle=cycle, 
+                                  min_dur=params['min_seg_dur'], 
+                                  reject_epoch=params['excl_epoch'], 
+                                  reject_artf=params['excl_event'])
+            
+            if not data.segments:
+                msg = 'No valid signal found.'
+                error_dialog = QErrorMessage(self)
+                error_dialog.setWindowTitle('Error fetching data')
+                error_dialog.showMessage(msg)
+                return
+            
+            ding = data.read_data(chans, group['ref_chan'], group['name'],
+                                  parent=self)
+            
+            if not ding:
+                self.parent.statusBar().showMessage('Process interrupted.')
+                return            
+            
+            self.parent.notes.detect_events(data[0]['data'], self.method, 
+                                            params, label=label)
 
             self.accept()
 
